@@ -139,7 +139,7 @@ c       from h1 with the zenith angle cost
        real*8 h1, h2, cost
 
        real*8 sint, costp, sintp
-       character*90 text
+       character*120 text
 
        sint = sqrt(1.d0 - cost**2)
        sintp = h1*sint /h2
@@ -150,7 +150,8 @@ c       from h1 with the zenith angle cost
           if(abs(1.d0-sintp**2) .lt. 1.d-1) then
              costp = 0.
           else
-             write(text, *) 'h1, h2, cost=', h1, h2, cost
+             write(text, *) 'h1, h2, cost=', h1, h2, cost, 
+     *          ' sintp=',sintp
              call cerrorMsg(text, 1)
              call
      *       cerrorMsg('h1,h2,cost invalid at clenbetwee2h', 0)
