@@ -38,6 +38,8 @@ c
          elseif(abs(proj.subcode) .eq. k0s) then
             ctau = tcks
          else
+            ctau = tckl  !rare ; some bug
+            proj.subcode = k0l
          endif
       elseif(proj.code .eq. kmuon) then
          ctau = tcmu
