@@ -48,7 +48,8 @@ c            default trace output is requested. Open disk file
       endif
 c
       call ciniASObs
-      if(abs(ObsPlane) .eq. horizontal ) then
+      if(abs(ObsPlane) .eq. horizontal  .or. 
+     *        ObsPlane .eq. spherical) then
          call csetObsZ(cxyz2det)
       elseif(abs(ObsPlane) .eq. perpendicular ) then
          call csetObsZ(cxyz2prim)
@@ -130,6 +131,7 @@ c
 
       external converter  ! cxyz2prim or cxyz2det
 c                           depending on |ObsPlane| =2, 1
+c                           or  cxyz2det for ObsPlane=3
       integer i
       record /coord/ temp
 c        compute z from the base detector.
@@ -206,16 +208,16 @@ c
       obas = index(gen, 'as') .gt. 0 .or.
      *   index(gen, 'lat') .gt. 0
 
-      if(obas ) then
-c          energy / nucleon for heavy
-         if(IncidentCopy.p.code .ge. kdeut .and.
-     *         IncidentCopy.p.code .le.  khvymax) then
-            ergpn =
-     *       IncidentCopy.p.fm.p(4)/Code2massN(IncidentCopy.p.code)
-         else
-            ergpn = IncidentCopy.p.fm.p(4)
-         endif
 
+c          energy / nucleon for heavy
+      if(IncidentCopy.p.code .ge. kdeut .and.
+     *     IncidentCopy.p.code .le.  khvymax) then
+         ergpn =
+     *        IncidentCopy.p.fm.p(4)/Code2massN(IncidentCopy.p.code)
+      else
+         ergpn = IncidentCopy.p.fm.p(4)
+      endif
+      if(obas ) then
                ! wait until electon  energy becomes < EasWait for AS
                ! generation. WaitRatio < 1 should be used when 1ry is 
                ! g or e.   (say, 0.01)
@@ -238,7 +240,11 @@ c          energy / nucleon for heavy
 
       if(ThinSampling) then
 c           thin sampling
-           Ethin = EthinRatio*ergpn
+         if(EthinRatio .lt. 0.) then
+            Ethin = -EthinRatio
+         else
+            Ethin = EthinRatio*ergpn
+         endif
       else
            Ethin = 0.
       endif
