@@ -9,7 +9,7 @@ c       one sampled priamary of which energy etc is given in
 c       variable in Prim.  Only /ptcl/ information is returned
 c       to the caller.
 c
-        subroutine csampPrimary(p)
+        subroutine csampPrimary(p, fin)
 c          p: /ptcl/  output.  energy, particle code, subcode,
 c                              charge, mass are set.
 c  
@@ -21,10 +21,35 @@ c
 #include  "Zprimaryv.h"
 
         record /ptcl/  p
-        call csampPrimary0(Prim)
-        call cconv_prim_e(Prim)   ! to total energy
-        p = Prim.particle
-        Prim.NoOfSamplings =  Prim.NoOfSamplings  + 1  ! update counter.
+        integer fin ! output. if no more sim. -->1 else 0
+
+        fin = 0
+        if( DestEventNo(2) .lt. 0) then
+           if(abs(DestEventNo(2)) .le.  Prim.NoOfSamplings) then
+              fin = 1
+           endif
+        endif
+        if(fin .eq. 0) then
+           call csampPrimary0(Prim)
+c          call cconv_prim_e(Prim)   ! to total energy
+           call cconv_prim_e(Prim.each(Prim.label), Prim.sampled_e, 
+     *      Prim.particle)         
+
+           p = Prim.particle
+           Prim.NoOfSamplings =  Prim.NoOfSamplings  + 1 ! update counter.
+           Prim.NoOfSampComp(Prim.label, 1) =
+     *          Prim.NoOfSampComp(Prim.label, 1) +1
+        endif
+        end
+c       **********************   
+        subroutine cupdtPrimC
+        implicit none
+#include  "Zptcl.h"
+#include  "Zprimary.h"
+#include  "Zprimaryv.h"
+
+        Prim.NoOfSampComp(Prim.label, 2) =
+     *       Prim.NoOfSampComp(Prim.label, 2) +1
         end
 c       ************************* see if geomagnetic cut or not.
         subroutine cifCutOff(icon)
@@ -39,7 +64,7 @@ c       ************************* see if geomagnetic cut or not.
          record /coord/ angleatOb
          record /track/ inc
 
-         real*8 p, rig, zen, azm, theta, rigcut
+         real*8 p, rig, zen, azm, theta,  prob, u
 
          call cqIncident(inc, angleatOb)
 
@@ -59,20 +84,21 @@ c              clock wise) The x-axis is XaxisFromSouth
 c            degrees from the south in counter clockwise.
 c            convert azm so that measured from the south
 c
-            azm = azm+ XaxisFromSouth
+            azm = mod(azm+ XaxisFromSouth, 360.d0)
             if(ZenValue .eq. 'cos') then
 c                table zenith is in cos
                zen = angleatOb.r(3)
             else
                zen = theta
             endif
-            call crigCut(azm, zen, rigcut)
             p = sqrt(inc.p.fm.p(4)**2 - inc.p.mass**2)
             rig = p/abs(inc.p.charge)
-            if(rig .lt. rigcut) then
-               icon = 1
-            else
+            call crigCut(azm, zen, rig, prob)
+            call rndc(u)
+            if(u .lt. prob) then
                icon = 0
+            else
+               icon = 1
             endif
          endif
          end
@@ -122,6 +148,7 @@ c              discard E> Emax
            enddo
  10        continue
 c
+           call rndc(u)
            i = each.no_of_seg 
            do while (u .gt. each.norm_inte(i))
                i = i - 1
@@ -129,13 +156,14 @@ c
 c             use i-th segment
            if(each.diff_or_inte .eq. 'd') then    
                ombeta = (1.d0 - each.beta(i))           
+               call rndc(u)
                if(abs(ombeta) .gt. 1.d-6) then
                   e1temp = each.energy(i)**ombeta
-                  call rndc(u)
                   p_or_e =( u* (each.energy(i+1)**ombeta - e1temp) +
      *             e1temp )** (1.d0/ombeta)
                else    
-                  p_or_e = each.energy(i+1)**u
+                  p_or_e =( each.energy(i)/each.energy(i+1) )**u
+     *                * each.energy(i+1)                  
                endif
            elseif(each.diff_or_inte .eq. 'i') then
                call rndc(u)
@@ -149,80 +177,155 @@ c             use i-th segment
            endif     
          endif  
          end
-         subroutine cconv_prim_e(prm)
-         implicit none
+        subroutine cconv_prim_e(comp, e_or_p,  aPtcl)
+        implicit none
+c          given energy or momentum as given in primary 
+c          specification is converted to total enregy.
 
 #include  "Zptcl.h"
 #include  "Zcode.h"
 #include  "Zprimary.h"
-         record /primaries/ prm
-         integer label, code, massn
+         record /component/ comp  ! input. primary component
+         real*8  e_or_p           ! input. energy or momentum as
+                                  !        specified in primary data file
+         record /ptcl/aPtcl       ! output. 'partcle' of the primary 
+         integer  code, massn
 c       
-         real*8 seinGeV  ! sampled primary in GeV
+         real*8 inGeV  ! p_or_e in  Gev
 
-         label = prm.label
 c
-         seinGeV= prm.sampled_e *   prm.each(label).togev
-         call cmkptc(prm.each(label).code,
-     *               prm.each(label).subcode,
-     *               prm.each(label).charge,
-     *               prm.particle)        
+         inGeV= e_or_p *   comp.togev
+         call cmkptc(comp.code,
+     *               comp.subcode,
+     *               comp.charge,
+     *               aPtcl)        
 
-         if(prm.each(label).code .eq. kmuon) then
+         if(comp.code .eq. kmuon) then
 c            give polarization for muon primary
             call csetMuonPol(0.d0)
          endif
 
-         if(prm.each(label).etype .eq. 'e/p' .or.
-     *      prm.each(label).etype .eq. 'e') then
-            prm.particle.fm.p(4) = seinGeV
+         if(comp.etype .eq. 'e/p' .or.
+     *      comp.etype .eq. 'e') then
+            aPtcl.fm.p(4) = inGeV
      *                  
-         elseif(prm.each(label).etype .eq.'p/p' .or.
-     *      prm.each(label).etype .eq. 'p')then
-            prm.particle.fm.p(4) = sqrt(prm.particle.mass**2 +
-     *             seinGeV**2)
-         elseif(prm.each(label).etype .eq. 'ke/p' .or.
-     *      prm.each(label).etype .eq. 'ke') then
-            prm.particle.fm.p(4) = seinGeV + prm.particle.mass
-         elseif(prm.each(label).etype .eq. 'e/n') then
-            code = prm.each(label).code
+         elseif(comp.etype .eq.'p/p' .or.
+     *      comp.etype .eq. 'p')then
+            aPtcl.fm.p(4) = sqrt(aPtcl.mass**2 +
+     *             inGeV**2)
+         elseif(comp.etype .eq. 'ke/p' .or.
+     *      comp.etype .eq. 'ke') then
+            aPtcl.fm.p(4) = inGeV + aPtcl.mass
+         elseif(comp.etype .eq. 'e/n') then
+            code = comp.code
             if(code .ge. kdeut .and. 
      *         code .le. khvymax) then
                call cghvm(code, massn)
-               prm.particle.fm.p(4) = massn * seinGeV
+               aPtcl.fm.p(4) = massn * inGeV
             else
-               prm.particle.fm.p(4) =  seinGeV
+               aPtcl.fm.p(4) =  inGeV
             endif
-         elseif(prm.each(label).etype .eq. 'ke/n') then
-            code = prm.each(label).code
+         elseif(comp.etype .eq. 'ke/n') then
+            code = comp.code
             if(code .ge. kdeut .and. 
      *         code .le. khvymax) then
                call cghvm(code, massn)
-               prm.particle.fm.p(4) = massn * seinGeV +
-     *            prm.particle.mass
+               aPtcl.fm.p(4) = massn * inGeV +
+     *            aPtcl.mass
             else
-               prm.particle.fm.p(4) = seinGeV +
-     *            prm.particle.mass
+               aPtcl.fm.p(4) = inGeV +
+     *            aPtcl.mass
             endif
-	 elseif(prm.each(label).etype .eq. 'p/n') then
-            code = prm.each(label).code
+	 elseif(comp.etype .eq. 'p/n') then
+            code = comp.code
             if(code .ge. kdeut .and. 
      *         code .le. khvymax) then
                call cghvm(code, massn)
-               prm.particle.fm.p(4) =
-     *      	  sqrt( (massn * seinGeV) **2 + 
-     *            prm.particle.mass **2)
+               aPtcl.fm.p(4) =
+     *      	  sqrt( (massn * inGeV) **2 + 
+     *            aPtcl.mass **2)
             else
-               prm.particle.fm.p(4) = sqrt(seinGeV**2 +
-     *            prm.particle.mass**2)
+               aPtcl.fm.p(4) = sqrt(inGeV**2 +
+     *            aPtcl.mass**2)
             endif
          else
-            write(*, *) ' energy type=',prm.each(label).etype,
-     *                  ' invalid. label=', label, ' symb =',
-     *                  prm.each(label).symb
+            write(*, *) ' energy type=', comp.etype,
+     *                  ' invalid. label=', comp.label, ' symb =',
+     *                  comp.symb
             stop 9999
          endif
         end
+c       *******************************************************
+        subroutine cconv_prim_e2(comp, E, e_or_p)
+        implicit none
+c         inverse of cconv_prim_e
+
+#include  "Zptcl.h"
+#include  "Zcode.h"
+#include  "Zprimary.h"
+         record /component/ comp  ! input. primary component
+         real*8  E                ! input. total energy of a particle
+                                  !   of 'comp' primary
+         real*8 e_or_p            ! output. e or p (/n, etc) as given 
+                                  !      in primary paticle file.
+         integer  code, massn
+c       
+         real*8 Ex         !  e_or_p in GeV
+
+         record /ptcl/aPtcl
+
+         call cmkptc(comp.code,
+     *               comp.subcode,
+     *               comp.charge,
+     *               aPtcl)        
+
+
+         if(comp.etype .eq. 'e/p' .or.
+     *      comp.etype .eq. 'e') then
+            Ex = E
+         elseif(comp.etype .eq.'p/p' .or.
+     *      comp.etype .eq. 'p')then
+            Ex = sqrt(E**2 - aPtcl.mass**2)
+
+         elseif(comp.etype .eq. 'ke/p' .or.
+     *      comp.etype .eq. 'ke') then
+            Ex = E - aPtcl.mass
+
+         elseif(comp.etype .eq. 'e/n') then
+            code = comp.code
+            if(code .ge. kdeut .and. 
+     *         code .le. khvymax) then
+               call cghvm(code, massn)
+               Ex = E/massn
+            else
+               Ex = E
+            endif
+         elseif(comp.etype .eq. 'ke/n') then
+            code = comp.code
+            if(code .ge. kdeut .and. 
+     *         code .le. khvymax) then
+               call cghvm(code, massn)
+               Ex =( E -   aPtcl.mass)/massn
+            else
+               Ex = E - aPtcl.mass
+            endif
+	 elseif(comp.etype .eq. 'p/n') then
+            code = comp.code
+            if(code .ge. kdeut .and. 
+     *         code .le. khvymax) then
+               call cghvm(code, massn)
+               Ex =sqrt( E**2 -   aPtcl.mass **2)/massn
+            else
+               Ex = sqrt(E**2 - aPtcl.mass**2)
+            endif
+         else
+            write(*, *) ' energy type=', comp.etype,
+     *                  ' invalid. label=', comp.label, ' symb =',
+     *                  comp.symb
+            stop 9999
+         endif
+        e_or_p = Ex/ comp.togev
+        end
 c       *******************************************************
 c       *  cqPrimE: inquire sampled primary energy or p
 c       *                    as it is
