@@ -171,6 +171,10 @@ c               update observation place
             elseif(abs(ObsPlane) .eq. perpendicular) then
                call cscalerProd(MovedTrack.vec.w, DcAtObsXyz,
      *               cosangle)
+            elseif(ObsPlane .eq. spherical) then
+               call cscalerProd(MovedTrack.vec.w, MovedTrack.pos,
+     *               cosangle)
+               cosangle = - cosangle
             else
                call 
      *         cerrorMsg('should not come here in tracking', 0)
@@ -182,14 +186,14 @@ c               update observation place
                MovedTrack.where = MovedTrack.where + 1
             endif
 c                  if more obs-site, stack current ptcl.
-            if(.not. Upgoing) then  ! downgoing
+            if(.not. Upgoing) then  ! incident is downgoing
                if(MovedTrack.where .gt. EndLevel) then
                   ! no need to stack; discard ptcl.
                else
                   MovedTrack.where =max( MovedTrack.where, 1)
                   call cpush(MovedTrack)
                endif
-            else    ! Upgoing
+            else    !incident is  Upgoing
                if(MovedTrack.where .lt.1 ) then
                    ! no need to stack
                else
@@ -382,6 +386,9 @@ c              convert kg/m2 into length
      *        IntInfArray(i).thickness .ne. Infty) then
                call cthick2len(h, TrackBefMove.vec.coszenith, 
      *         IntInfArray(i).thickness, leng, t, jcut)
+               if(leng .le. 0.) then
+                  leng = 1.d-3
+               endif
             else
                leng = Infty
             endif
